/*
**  CXSC is a C++ library for eXtended Scientific Computing (V 2.5.4)
**
**  Copyright (C) 1990-2000 Institut fuer Angewandte Mathematik,
**                          Universitaet Karlsruhe, Germany
**            (C) 2000-2014 Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* CVS $Id: z_zsta.c,v 1.21 2014/01/30 17:24:18 cxsc Exp $ */

/****************************************************************/
/*                                                              */
/*      Filename        : z_zsta.c                              */
/*                                                              */
/*      Entries         : a_cinv z_zsta(crl,cil,cru,ciu)        */
/*                        dotprecision crl,cil,cru,ciu;         */
/*                                                              */
/*      Arguments       : crl=dotprecision value(real lower)    */
/*                        cil=dotprecision value(imag lower)    */
/*                        cru=dotprecision value(real upper)    */
/*                        ciu=dotprecision value(imag upper)    */
/*                                                              */
/*      Description     : Convert complex interval              */
/*                                                              */
/****************************************************************/

#ifndef ALL_IN_ONE
#ifdef AIX
#include "/u/p88c/runtime/o_defs.h"
#else
#include "o_defs.h"
#endif
#define local
#endif

#ifdef LINT_ARGS
local a_cinv z_zsta(dotprecision crl,dotprecision cil,dotprecision cru,
                    dotprecision ciu)
#else
local a_cinv z_zsta(crl,cil,cru,ciu)

dotprecision crl;
dotprecision cil;
dotprecision cru;
dotprecision ciu;
#endif
        {
        a_cinv res;

        E_TPUSH("z_zsta")

        res.RE.INF = d_stad(crl);
        res.IM.INF = d_stad(cil);
        res.RE.SUP = d_stau(cru);
        res.IM.SUP = d_stau(ciu);

        E_TPOPP("z_zsta")
        return(res);
        }





