/*
**  CXSC is a C++ library for eXtended Scientific Computing (V 2.5.4)
**
**  Copyright (C) 1990-2000 Institut fuer Angewandte Mathematik,
**                          Universitaet Karlsruhe, Germany
**            (C) 2000-2014 Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* CVS $Id: b_subu.c,v 1.21 2014/01/30 17:24:05 cxsc Exp $ */

/****************************************************************/
/*                                                              */
/*      Filename        : b_subu.c                              */
/*                                                              */
/*      Entry           : void b_subu(a,b,bin,c,bout);          */
/*                        a_btyp a,b,*c;                        */
/*                        a_intg bin,*bout;                     */
/*                                                              */
/*      Arguments       : a = first operand of subtraction      */
/*                        b = second operand of subtraction     */
/*                        bin = borrow in                       */
/*                        c = a-b-bin                           */
/*                        bout = borrow out                     */
/*                                                              */
/*      Description     : Subtraction of a_btyp digits with     */
/*                        borrow                                */
/*                                                              */
/****************************************************************/

#ifndef ALL_IN_ONE
#ifdef AIX
#include "/u/p88c/runtime/o_defs.h"
#else
#include "o_defs.h"
#endif
#define local
#endif

#ifdef LINT_ARGS
local void b_subu(a_btyp a,a_btyp b,a_intg bin,a_btyp *c,a_intg *bout)
#else
local void b_subu(a,b,bin,c,bout)

a_btyp a;
a_btyp b;
a_intg bin;
a_btyp *c;
a_intg *bout;
#endif
        {
#if C_P_2
        *c = a-b-bin;
        *bout = (b>a || (b==a && bin)) ? 1 : 0;
#else
        /* b+bin produce carry => (c-bout) = (a-1)              */
        if ( (b==MAX_BASETYPE) && bin ) {
                *bout = 1;
                *c = a;
                return;
                }

        b += bin;

        /* determine borrow and difference                      */
        if (a>=b) {
                *bout = 0;
                *c = a-b;
                }
        else {
                *bout = 1;
                *c = ~(b-a)+1;
                }
#endif

        return;
        }





